package org.jfedor.subwaybuddies;
import android.app.Service;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationManager;
import android.os.IBinder;
import android.util.Log;


public class ScannerService extends Service {
	
	private BluetoothAdapter mBtAdapter;
	boolean mBtInitiallyEnabled;
	private SQLiteDatabase mDb;
	long scanId = 0;

	@Override
	public IBinder onBind(Intent intent) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void onCreate() {
		super.onCreate();
		
		SQLiteOpenHelper openHelper = new ScanDbOpenHelper(this);
		mDb = openHelper.getWritableDatabase();
		
		mBtAdapter = BluetoothAdapter.getDefaultAdapter();
		
		IntentFilter filter = new IntentFilter(BluetoothDevice.ACTION_FOUND);
        this.registerReceiver(mReceiver, filter);
        
        filter = new IntentFilter(BluetoothAdapter.ACTION_DISCOVERY_FINISHED);
        this.registerReceiver(mReceiver, filter);
		
        filter = new IntentFilter(BluetoothAdapter.ACTION_STATE_CHANGED);
        this.registerReceiver(mReceiver, filter);
        
		if (mBtAdapter.isEnabled()) {
			mBtInitiallyEnabled = true;
			startScan();
		} else {
			mBtInitiallyEnabled = false;
			mBtAdapter.enable();
		}
	}
	
	void startScan() {
		if (scanId == 0) {
			ContentValues values = new ContentValues();
			values.put("starttime", (int)(System.currentTimeMillis()*0.001));
			LocationManager lm = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
			String provider = lm.getBestProvider(new Criteria(), true);
			Location loc = lm.getLastKnownLocation(provider);
			values.put("lat", Double.toString(loc.getLatitude()));
			values.put("lon", Double.toString(loc.getLongitude()));
			scanId = mDb.insert("scan", null, values); // we probably shouldn't be doing i/o on the main thread
		}
		if (!mBtAdapter.isDiscovering()) {
			mBtAdapter.startDiscovery();
		}
	}

	@Override
	public void onDestroy() {
		// TODO Auto-generated method stub
		super.onDestroy();
	}

	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		
		return Service.START_STICKY;
	}

    private final BroadcastReceiver mReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            
            if (BluetoothAdapter.ACTION_STATE_CHANGED.equals(action)) {
            	if (mBtAdapter.getState() == BluetoothAdapter.STATE_ON) {
            		startScan();
            	} // else?
            } else if (BluetoothDevice.ACTION_FOUND.equals(action)) {
                BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                Log.i("BT", "found: "+ device.getAddress() + " " + device.getName());
                ContentValues values = new ContentValues();
                values.put("scan_id", scanId);
                values.put("sightingtime", (int)(System.currentTimeMillis()*0.001));
                values.put("addr", device.getAddress());
                values.put("name", device.getName());
                mDb.insert("sighting", null, values);
            } else if (BluetoothAdapter.ACTION_DISCOVERY_FINISHED.equals(action)) {
            	Log.i("BT", "finished");
            	unregisterReceiver(mReceiver);
            	if (!mBtInitiallyEnabled) {
            		mBtAdapter.disable();
            	}
            	mDb.close();
            	stopSelf();
            }
        }
    };
}
